/**
***     Blue Chip Technology    Dynamic Link Library
***
***
***     BCdefn.h                Manifest Constants and Internal
***                             Function Prototypes
***
***     This module contains manifest constants, type definitions and
***     function prototypes needed to build the DLL.
***
***     This is an internal module and NOT intended for applications
***     builders (see BLUECHIP.h)
***
***     Author          :       Andrew Thomson,
***                             Nigel Roberts,
***                             Stuart Grace.
***
***     Version History :
***
***   	BLUECHIP 1.00   2/11/93  First Version. Not released.
***
***              1.01   1/2/94   Rejig of the way interrupts work.
***		                 Board busy semaphore instead of DLL wide.
***
***              1.02   19/3/94  Added "undocumented" function for DMA buffers
***
***				 1.06   6/11/94  Added AIP-8D support and fixed one or two
***                              bugs.
***				 1.07	9/2/95	 Modification for AOP_12D, added initialisation
***								 code for this board.	
**/ 

#ifndef __BCDEFN_H__
#define __BCDEFN_H__

/**	These would be in windows.h, but we aren't
**/
#ifndef _WINDLL
typedef short			BOOL;
typedef unsigned char	BC_BYTE;
typedef unsigned short	BC_WORD;
typedef unsigned long	BC_DWORD;
typedef void __huge *	HANDLE;

#define lstrlen			_fstrlen
#define lstrcpyn		_fstrncpy
#define TRUE			((BOOL)1)
#define FALSE			((BOOL)0)

#define LHND			0
#define GPTR			0
#endif

/**     ---------------------------------------------------------
***     Manifest constants
***     ---------------------------------------------------------
**/

#define SZ_VERSION_DESCRIPTION  "1.07 Release 4, PCI Mods"






/**     'nOffset' Values for SetBoardValue and GetBoardValue functions
***     used to determine which field of the BCBOARD structure is to be
***     accessed
**/
#define GBV_BASEADDRESS 1  /* The base address of the board was installed */
#define GBV_BOARDTYPE   2  /* The defined code for the board type         */
#define GBV_DMACHANNEL  3  /* The DMA channel selected for the board.     */
#define GBV_GAIN        4  /* The software selected input gain            */
#define GBV_IRQ         5  /* The Interrupt level selected for the board  */
#define GBV_MODE        6  /* The software selected mode for the board.   */
#define GBV_BOARD		7




/**     Port direction codes ( see structure BCTBOARD:anPorts )
**/
#define PD_UNUSED       0    /* Port not available */
#define PD_OUTPUT       1    /* All 8 bits are output   */
#define PD_INPUT        2    /* All 8 bits are input    */
#define PD_INPUT_LOW    3    /* bits 0-3 input,  bits 4-7 output */
#define PD_INPUT_HIGH   4    /* bits 0-3 output, bits 4-7 input  */
#define PD_UNCONFIGURED 5    /* Applc has not yet configured this port */

/**		New PCI definitions
**/
#define PD0_OUTPUT		6	/* Mode 0, all 8 bits output */
#define PD1_OUTPUT		7	/* Mode 1, all 8 bits output */
#define PD2_OUTPUT		8	/* Mode 2, all 8 bits output */
#define PD0_INPUT		9	/* Mode 0, all 8 bits input */
#define PD1_INPUT		10	/* Mode 1, all 8 bits input */
#define PD2_INPUT		11	/* Mode 2, all 8 bits input */

#define MIN_BASEADDR		0x00
#define MIN_PCIBASEADDR		0x00
#define MAX_BASEADDR		0x03FC
#define MAX_PCIBASEADDR		0xFFFF
#define MAX_IRQ				7
#define MAX_PCIIRQ			15
#define MIN_IRQ				2





/**     Miscellaneous constants
**/
#define MAX_BOARDS      16     /* Limits number of boards we can request */
#define MAX_PORTS       18     /* Max number of digital ports on board   */
#define UNUSED_HANDLE   -1     /* Used to denote empty BCTBOARD struct   */
#define CONVERSIONTIME  3e-05  /* Approx time in sec for conversion/settling */

#define MAX_SCAN_CHANS  16

#define OP_INPUT  0     /* Used with DMA and IRQ operations - Input */
#define OP_OUTPUT 1     /* Used with DMA and IRQ operations - Output */



#define AIP_8D_MAX_FREQ 80000.0F        /* Leave atleast 12us to convert */


/**     Error & status codes returned by the "undocumented" function
***     BCTReserveDMABuffers. The program BCT_DMA.EXE translates these
***     into text messages.
**/

#define ONE_BUFFER_OK           900     /* One buffer allocated         */
#define TWO_BUFFERS_OK          901     /* Both buffers allocated       */
#define ONLY_FIRST_OK	        902     /* Only one could be obtained	*/
#define OUT_OF_VIRTUAL_MEM      903     /* insufficient global memory   */
#define PAGES_NOT_ADJACENT      904     /* memory messed up by paging   */
#define DMA_IN_PROGRESS         905     /* DMA has already started 	*/
#define MEMORY_NOT_REACHABLE    906     /* Free mem is too high for DMA	*/





/**     ---------------------------------------------------------
***     Structure declaration
***     ---------------------------------------------------------
**/


/**   structure BCTCHANNEL
***
***   This structure holds the range information for a single analogue
***   channel. This information may be used to scale the ADC/DAC data
***   in to real world values.
***
***   It is used internally within the DLL and is not available to the
***   calling application.
**/
typedef struct BCTCHANNELtag {

	float   flMinimum;      /** Value represented by ADC Value 0    **/
	float   flMaximum;      /** Value represented by ADC value 4095 **/

} BCTCHANNEL,  *PBCTCHANNEL;





/**   structure BCBOARD
***
***   This structure holds the information required to access a single
***   Blue Chip data acquistion board.
***
***   It is used internally within the DLL and is not available to the
***   calling application.
***
**/
typedef struct BCTBOARDtag {

	short   nHandle;        /** Handle assigned by BCTRequestBoard
				*** a handle of -1 is used to indicate
				*** that the does not contain valid
				*** board information.
				**/

	short   nType;          /** Board Type code ( see BCDefn.h for full
				*** list of supported boards )
				**/

	short   nBaseAddress;   /** User configured 'Base address' of the
				*** board.
				**/

	short   nIRQ;           /** User configured IRQ level provided for
				*** boards with interrupt support. IRQ levels
				*** 2 to 7 are permitted by the hardware.
				**/

	short   nBoard;           /** PCI Board Number
				*** boards with interrupt support. IRQ levels
				*** 2 to 7 are permitted by the hardware.
				**/

	short   nDMAChannel;    /** User configured DMA Channel provided for
				*** boards with DMA support.
				**/

	short   nGain;          /** 'Input gain' setting for boards which
				*** support software gains.
				**/

        short   nMode;          /** Mode of board operation. E.g. single
                                *** ended or differential.
                                **/


        short   bBusyInBackground;      /** The board is being used in
                                        *** the background doing i/p or o/p
                                        *** under DMA or IRQ control.
                                        **/

	short   nOperation;             /* OP_INPUT or OP_OUTPUT */


	short   anPorts [ MAX_PORTS ];    /** Holds Port code for each
					  *** possible digital port on
					  *** the board. Codes are :
					  ***
					  *** PD_INPUT     input port
					  ***
					  *** PD_OUTPUT    output port
					  ***
					  *** PD_INPUTHIGH 4 high bits input
					  ***              only
					  ***
					  *** PD_INPUTLOW  4 low bits input
					  ***              only
					  ***
					  *** PD_UNCONFIGURED Not configured
					  ***                 by the user yet.
					  ***
					  *** PD_UNUSED    This port number
					  ***              is not supported by
					  ***              this board
					  **/

	short   anPortStates[ MAX_PORTS ];/* Current state of o/ps on port */


	HANDLE  hBCTChans;      /** Handle of BCTCHANNEL array.**/

	PBCTCHANNEL pBCTChans;	/** pointer to an array of BCTCHANNEL
				*** structures.
				**/

} BCTBOARD, *PBCTBOARD;



/**     structure IRQINFO
***
***     This structure is predominately used by the interrupt service
***     routine and its associated functions in module BCIRQ.c
***
***     It also available to BLUECHIP.c where it may be needed if the
***     ISR is still installed when the DLL is being closed.
***
**/
typedef struct IRQINFOtag {

     short      nLevel;                 /* IRQ Level for our ISR           */
     short      nOperation;             /* I/P or O/P                      */
     short      nHandle;                /* Board's handle                  */
     short      nBoardType;             /* Type of board generating IRQs   */
     BOOL       bIntAlreadyEnabled;     /* Mask was off before ISR started */
     BOOL       bVectorHooked;          /* Our ISR is installed            */
     BC_BYTE       byFreqProgCode;         /* Byte to set reqd frequency      */
     short      nFreqRegister;          /* Register to send above byte     */
     short      nBaseAddress;           /* Base address of board           */
     long       lScansRequired;         /* Scans required by ISR           */
     HANDLE     hData; 					/* handle of data array            */
     short _huge *lpanData;             /* Pointer into data buffer        */
     BC_DWORD      ulDataIndex;
     BC_DWORD      ulBytes;                /* size of data array              */
     short 	anChans [ MAX_SCAN_CHANS ];  /* Array of chans nos.        */
     short      nLength;                /* Num chans in array              */
     short      fRepeat;                /* Repeat flag                     */
     short      nChanIndex;             /* index into channel array        */

     /** Lastly the existing ISR vector which we are replacing
     **/
//     void (__cdecl __interrupt __far *fpfnOldHandler) ( void );
     void (__cdecl __interrupt __far *fpfnOldHandler) ( );
} IRQINFO;



/**	Variables changed by ISR
**/
typedef struct ISRVARIABLEStag {

	BOOL 	bInISR;			/* ISR semaphore		 */
	BOOL 	bCantKeepUp;		/* Detects new IRQ whilst in ISR */
	long 	lScansCollected;	/* scans so far; updated by ISR	 */

} ISRVARS;


#define MAX_ISRS        2       /** Number of ISRs 1 i/p and 1 o/p
                                **/

/*	Data structures and constants for the PCI Interrupt handler - 1 set for
	each board type
*/

//	Setup for PCI_PIO
#define MAX_PCI_PIO_ISR_SOURCES		7	
#define MAX_PCI_PIO_BOARDS			3

typedef struct _PCI_PIO_ISR_DATA {
	BC_DWORD	*A_IBF_Semaphore;
	BC_DWORD	*A_OBF_Semaphore;
	BC_DWORD	*B_IBF_Semaphore;
	BC_DWORD	*B_OBF_Semaphore;
	BC_DWORD	*TIMER0_Semaphore;
	BC_DWORD	*TIMER1_Semaphore;
	BC_DWORD	*TIMER2_Semaphore;
	BC_WORD		Irq;
	BOOL		bIntAlreadyEnabled;     /* Mask was off before ISR started */
	BOOL		bVectorHooked;          /* Our ISR is installed            */
	BOOL		bInISR;					/* ISR semaphore		 */
	BOOL		bCantKeepUp;			/* Detects new IRQ whilst in ISR */
	BC_BYTE		IrqVec;			// Vector to attach the Interrupt handling routine to
	BC_BYTE		IrqEna;			// this bit when 0 enable the Interrupt, 1 to disable
	short		BaseAddress;

	void (__cdecl __interrupt __far *fpfnOldHandler) ( );
} PCI_PIO_ISR_DATA;

//	Interrupt Source Index for the ISR_TABLE
#define PCI_PIO_ISR_AIBF				0
#define PCI_PIO_ISR_AOBF				1
#define PCI_PIO_ISR_BIBF				2
#define PCI_PIO_ISR_BOBF				3
#define PCI_PIO_ISR_TIMER0				4
#define PCI_PIO_ISR_TIMER1				5
#define PCI_PIO_ISR_TIMER2				6

//	Action codes for the ISR_TABLE
#define PCI_PIO_ISR_NO_ACTION			0
#define PCI_PIO_ISR_SIGNAL_SEMAPHORE	1

// Program offsets for PCI_PIO
#define PCI_PIO_PIO1_A					0
#define PCI_PIO_PIO1_B					1
#define PCI_PIO_PIO1_C					2
#define PCI_PIO_PIO1_CTRL				3
#define PCI_PIO_PIO2_A					4
#define PCI_PIO_PIO2_B					5
#define PCI_PIO_PIO2_C					6
#define PCI_PIO_PIO2_CTRL				7
#define PCI_PIO_TIMER0					8
#define PCI_PIO_TIMER1					9
#define PCI_PIO_TIMER2					10
#define PCI_PIO_TIMER_CTRL				11
#define PCI_PIO_INT_ENABLE				12
#define PCI_PIO_INT_STATUS				13
#define PCI_PIO_COUNTER_CTRLA			14
#define PCI_PIO_COUNTER_CTRLB			15

//	PCI_PIO Interrupt Enables
#define PCI_PIO_ENABLE_AC0				0x01
#define PCI_PIO_ENABLE_AC3				0x02
#define PCI_PIO_ENABLE_BC0				0x04
#define PCI_PIO_ENABLE_BC3				0x08
#define PCI_PIO_ENABLE_TIMER0			0x10
#define PCI_PIO_ENABLE_TIMER1			0x20
#define PCI_PIO_ENABLE_TIMER2			0x40

// 8254 Bit Definitions
#define PCI_8254_CTRL_MODE0				0x00
#define PCI_8254_CTRL_MODE1				0x02
#define PCI_8254_CTRL_MODE2				0x04
#define PCI_8254_CTRL_MODE3				0x06
#define PCI_8254_CTRL_MODE4				0x08
#define PCI_8254_CTRL_MODE5				0x0a
#define PCI_8254_CTRL_BCD				0x01
#define PCI_8254_SC0					0x00
#define PCI_8254_SC1					0x40
#define PCI_8254_SC2					0x80
#define PCI_8254_READBACK				0xc0
#define PCI_8254_LATCH					0x00
#define PCI_8254_MSB					0x100000L
#define PCI_8254_LSB					0x200000L
#define PCI_8254_LSBMSB					0x300000L

// PCI_PIO Interrupt Source Settings
#define PCI_PIO_CLKSRC_OSC				0x00000000L
#define PCI_PIO_CLKSRC_PB				0x01000000L
#define PCI_PIO_CLKSRC_CHAIN			0x02000000L
#define PCI_PIO_CLKSRC_PC				0x03000000L

#define PCI_PIO_CLK0_INTONHIGH			0x40


/**     ---------------------------------------------------------
***     Function Prototypes
***     ---------------------------------------------------------
**/


/**     Function prototypes : Module BCmanage.c
**/

short AddNewBoardToTable   ( PBCTBOARD pNewboard, short far *lpnHandle );
short RemoveBoardFromTable ( short nHandleOfBoardToRelease );

BOOL  IsHandleValid   ( short nBoardHandle );
short GetBoardValue   ( short nBoardHandle, short nOffset );
short SetBoardValue   ( short nBoardHandle, short nOffset, short nValue );
short GetChannelRange ( short nBoardHandle, short nChannel,
			float *pflMinValue, float *pflMaxValue );
short SetChannelRange ( short nBoardHandle, short nChannel,
				float flMinValue, float flMaxValue );

short GetPortDirection ( short nBoardHandle, short nPort );
short SetPortDirection ( short nBoardHandle, short nPort, short nPortCode );

short GetPortState ( short nBoardHandle, short nPort );
short SetPortState ( short nBoardHandle, short nPort, short nPortState );

void InitializePortDirections ( short nHandle );
void InitializePortStates ( short nHandle );


short   IsBoardBusy ( short nBoardHandle );
short   SetBoardBusy ( short nBoardHandle, short fBusy, short nOperation );
short   GetBoardOperation ( short nBoardHandle );



/**     Function prototypes : Module BCanalog.c
**/

short AIP_24SetGain      ( short nHandle, short nGain );
short ADC_44DSetGain     ( short nHandle, short nGain );
short AIP_24ReadChannel  ( short nHandle, short nChannel );
short AIP_8DReadChannel  ( short nHandle, short nChannel );
short ADC_42ReadChannel  ( short nHandle, short nChannel );
short ADC_44DReadChannel ( short nHandle, short nChannel );
short AOP_8WriteChannel  ( short nHandle, short nChannel, short nDAC );
short AOP_12DWriteChannel( short nHandle, short nChannel, short nDAC );
short ADC_42WriteChannel ( short nHandle, short nChannel, short nDAC );
short ADC_44DWriteChannel( short nHandle, short nChannel, short nDAC );
void  AOP_12DBoardInitialise(short nBaseAddress);



/**     Function prototypes : Module BCdigit.c
**/

void PIO_96ConfigurePorts   ( short nHandle );
void PIO_48DBConfigurePorts ( short nHandle );
void PIO_144PConfigurePorts ( short nHandle );
void ADC_42ConfigurePorts   ( short nHandle );
void ADC_44DConfigurePorts  ( short nHandle );
void AOP_12DConfigurePorts  ( short nHandle );
void PCI_PIOConfigurePorts 	( short nHandle );

BC_BYTE Make8255ControlByte ( short nPortADir, short nPortBDir, short PortCDir );
BC_BYTE  MakePCI8255ControlByte(short nPortADir, short nPortBDir,
							 	short nPortCLow, short nPortCHigh );

short DIP_24ReadPort   ( short nHandle, short nPortNumber );
short DIO_48ReadPort   ( short nHandle, short nPortNumber );
short DCM_16ReadPort   ( short nHandle, short nPortNumber );
short PIO_96ReadPort   ( short nHandle, short nPortNumber );
short PIO_48DBReadPort ( short nHandle, short nPortNumber );
short PIO_144PReadPort ( short nHandle, short nPortNumber );
short ADC_44DReadPort  ( short nHandle, short nPortNumber );
short ADC_42ReadPort   ( short nHandle, short nPortNumber );
short AOP_12DReadPort   ( short nHandle, short nPortNumber );

void ADC_42WritePort   ( short nHandle, short nPortNumber, short nData );
void DOP_24WritePort   ( short nHandle, short nPortNumber, short nData );
void DIO_48WritePort   ( short nHandle, short nPortNumber, short nData );
void DCM_16WritePort   ( short nHandle, short nPortNumber, short nData );
void PIO_96WritePort   ( short nHandle, short nPortNumber, short nData );
void RB_8WritePort     ( short nHandle, short nPortNumber, short nData );
void PIO_48DBWritePort ( short nHandle, short nPortNumber, short nData );
void PIO_144PWritePort ( short nHandle, short nPortNumber, short nData );
void ADC_44DWritePort  ( short nHandle, short nPortNumber, short nData );
void AOP_12DWritePort  ( short nHandle, short nPortNumber, short nData );
void PCI_PIOWritePort  ( short nHandle, short nPortNumber, short nData );
short PCI_PIOReadPort  ( short nHandle, short nPortNumber );





/**     Function prototypes : Module BCcount.c
**/

void  DCM_16SetSquareWave ( short nHandle, short nCounter, long lPulseCount );
long  DCM_16ReadCounter ( short nHandle, short nCounter, BOOL bReset );





/**     Function prototypes : Module BCirq.c
**/

//void OurSetVect ( BC_WORD wIRQ, void ( _cdecl _interrupt _far *Handler) () );

short SetAIP_8DPacer ( short nHandle, short nChans, float flReqFreq );




/**     Function prototypes : Module BCdma.c
**/

void  InitialiseDMAStructures ( void );
void  ReleaseDMABuffers ( void );


/**     This is the "undocumented" EXPORT function in BCDMA.C which is
***     used by the BCT_DMA program to reserve DMA buffers in contiguous
***     physical memory. End users should run the BCT_DMA program, not
***     call the function directly.
**/

short _export _far _pascal BCTReserveDMABuffers ( short nBuffers );

#endif
